@@ -32,6 +32,7 @@
 #include "samplv1_reverb.h"
 
 #include "samplv1_config.h"
+#include "samplv1_controls.h"
 #include "samplv1_programs.h"
 
 
@@ -716,15 +717,15 @@ class samplv1_impl
 {
 public:
 
-	samplv1_impl(samplv1 *pSampl, uint16_t iChannels, uint32_t iSampleRate);
+	samplv1_impl(samplv1 *pSampl, uint16_t nchannels, float srate);
 
 	~samplv1_impl();
 
-	void setChannels(uint16_t iChannels);
+	void setChannels(uint16_t nchannels);
 	uint16_t channels() const;
 
-	void setSampleRate(uint32_t iSampleRate);
-	uint32_t sampleRate() const;
+	void setSampleRate(float srate);
+	float sampleRate() const;
 
 	void setSampleFile(const char *pszSampleFile);
 	const char *sampleFile() const;
@@ -735,6 +736,7 @@ public:
 	void setParamValue(samplv1::ParamIndex index, float fValue);
 	float paramValue(samplv1::ParamIndex index) const;
 
+	samplv1_controls *controls();
 	samplv1_programs *programs();
 
 	void process_midi(uint8_t *data, uint32_t size);
@@ -775,12 +777,13 @@ protected:
 private:
 
 	samplv1_config   m_config;
+	samplv1_controls m_controls;
 	samplv1_programs m_programs;
 
-	uint16_t m_iChannels;
-	uint32_t m_iSampleRate;
+	uint16_t m_nchannels;
+	float    m_srate;
 
-	samplv1_ctl m_ctl;
+	samplv1_ctl m_ctl1;
 
 	samplv1_gen m_gen1;
 	samplv1_dcf m_dcf1;
@@ -838,8 +841,8 @@ samplv1_voice::samplv1_voice ( samplv1_impl *pImpl ) :
 // engine constructor
 
 samplv1_impl::samplv1_impl (
-	samplv1 *pSampl, uint16_t iChannels, uint32_t iSampleRate )
-	: m_programs(pSampl)
+	samplv1 *pSampl, uint16_t nchannels, float srate )
+	: gen1_sample(pSampl), m_controls(pSampl), m_programs(pSampl)
 {
 	// null sample.
 	m_gen1.sample0 = 0.0f;
@@ -877,18 +880,19 @@ samplv1_impl::samplv1_impl (
 	m_del.bpmsync0 = 0.0f;
 	m_del.bpm0 = 0;
 
-	// load programs database...
+	// load controllers & programs database...
+	m_config.loadControls(&m_controls);
 	m_config.loadPrograms(&m_programs);
 
 	// number of channels
-	setChannels(iChannels);
+	setChannels(nchannels);
 
 	// parameters
 	for (int i = 0; i < int(samplv1::NUM_PARAMS); ++i)
 		setParamPort(samplv1::ParamIndex(i));
 
 	// set default sample rate
-	setSampleRate(iSampleRate);
+	setSampleRate(srate);
 
 	// reset all voices
 	allControllersOff();
@@ -920,9 +924,9 @@ samplv1_impl::~samplv1_impl (void)
 }
 
 
-void samplv1_impl::setChannels ( uint16_t iChannels )
+void samplv1_impl::setChannels ( uint16_t nchannels )
 {
-	m_iChannels = iChannels;
+	m_nchannels = nchannels;
 
 	// deallocate flangers
 	if (m_flanger) {
@@ -952,33 +956,33 @@ void samplv1_impl::setChannels ( uint16_t iChannels )
 
 uint16_t samplv1_impl::channels (void) const
 {
-	return m_iChannels;
+	return m_nchannels;
 }
 
 
-void samplv1_impl::setSampleRate ( uint32_t iSampleRate )
+void samplv1_impl::setSampleRate ( float srate )
 {
 	// set internal sample rate
-	m_iSampleRate = iSampleRate;
+	m_srate = srate;
 
 	// update waves sample rate
-	gen1_sample.setSampleRate(m_iSampleRate);
-	lfo1_wave.setSampleRate(m_iSampleRate);
+	gen1_sample.setSampleRate(m_srate);
+	lfo1_wave.setSampleRate(m_srate);
 
 	updateEnvTimes();
 }
 
 
-uint32_t samplv1_impl::sampleRate (void) const
+float samplv1_impl::sampleRate (void) const
 {
-	return m_iSampleRate;
+	return m_srate;
 }
 
 
 void samplv1_impl::updateEnvTimes (void)
 {
 	// update envelope range times in frames
-	const float srate_ms = 0.001f * float(m_iSampleRate);
+	const float srate_ms = 0.001f * m_srate;
 
 	float envtime_msecs = 10000.0f * m_gen1.envtime0;
 	if (envtime_msecs < MIN_ENV_MSECS)
@@ -1099,6 +1103,34 @@ void samplv1_impl::setParamPort ( samplv1::ParamIndex index, float *pfParam )
 	case samplv1::DYN1_LIMITER:   m_dyn.limiter      = pfParam; break;
 	default: break;
 	}
+
+	// check null connections.
+	if (pfParam == &s_fDummy)
+		return;
+
+	// reset ramps after port (re)connection.
+	switch (index) {
+	case samplv1::OUT1_VOLUME:
+	case samplv1::DCA1_VOLUME:
+		m_vol1.reset(
+			m_out1.volume,
+			m_dca1.volume,
+			&m_ctl1.volume,
+			&m_aux1.volume);
+		break;
+	case samplv1::OUT1_WIDTH:
+		m_wid1.reset(
+			m_out1.width);
+		break;
+	case samplv1::OUT1_PANNING:
+		m_pan1.reset(
+			m_out1.panning,
+			&m_ctl1.panning,
+			&m_aux1.panning);
+		break;
+	default:
+		break;
+	}
 }
 
 
@@ -1188,7 +1220,7 @@ void samplv1_impl::setParamValue ( samplv1::ParamIndex index, float fValue )
 {
 	float *pfParamPort = paramPort(index);
 	if (pfParamPort)
-		*pfParamPort = fValue;
+		*pfParamPort = fValue; // LV2:BUG?
 }
 
 
@@ -1203,173 +1235,190 @@ float samplv1_impl::paramValue ( samplv1::ParamIndex index ) const
 
 void samplv1_impl::process_midi ( uint8_t *data, uint32_t size )
 {
-	// check data size (#1)
-	if (size < 2)
-		return;
+	for (uint32_t i = 0; i < size; ++i) {
 
-	// channel filter
-	const int channel = (data[0] & 0x0f) + 1;
+		// channel status
+		const int channel = (data[i] & 0x0f) + 1;
+		const int status  = (data[i] & 0xf0);
 
-	const int ch = int(*m_def.channel);
-	const int on = (ch == 0 || ch == channel);
+		// channel filter
+		const int ch = int(*m_def.channel);
+		const int on = (ch == 0 || ch == channel);
 
-	if (!on)
-		return;
+		// all system common/real-time ignored
+		if (status == 0xf0)
+			continue;
 
-	// note on
-	const int status = (data[0] & 0xf0);
-	const int key    = (data[1] & 0x7f);
+		// check data size (#1)
+		if (++i >= size)
+			break;
+
+		const int key = (data[i] & 0x7f);
+
+		// program change
+		if (status == 0xc0) {
+			if (on) m_programs.prog_change(key);
+			continue;
+		}
 
-	// program change
-	if (status == 0xc0)
-		m_programs.prog_change(key);
-	else
-	if (status == 0xd0) {
 		// channel aftertouch
-		m_ctl.pressure = float(key) / 127.0f;
-	}
+		if (status == 0xd0) {
+			if (on) m_ctl1.pressure = float(key) / 127.0f;
+			continue;
+		}
 
-	// check data size (#2)
-	if (size < 3)
-		return;
+		// check data size (#2)
+		if (++i >= size)
+			break;
 
-	const int value  = (data[2] & 0x7f);
+		// channel filter
+		if (!on)
+			continue;
 
-	// note on
-	if (status == 0x90 && value > 0) {
-		samplv1_voice *pv;
-		// mono voice modes
-		if (*m_def.mono > 0.0f) {
-			for (pv = m_play_list.next(); pv; pv = pv->next()) {
-				if (pv->note >= 0
-					&& pv->dca1_env.stage != samplv1_env::Release) {
-					m_dcf1.env.note_off_fast(&pv->dcf1_env);
-					m_lfo1.env.note_off_fast(&pv->lfo1_env);
-					m_dca1.env.note_off_fast(&pv->dca1_env);
-					m_notes[pv->note] = 0;
-					pv->note = -1;
+		const int value = (data[i] & 0x7f);
+
+		// note on
+		if (status == 0x90 && value > 0) {
+			samplv1_voice *pv;
+			// mono voice modes
+			if (*m_def.mono > 0.0f) {
+				for (pv = m_play_list.next(); pv; pv = pv->next()) {
+					if (pv->note >= 0
+						&& pv->dca1_env.stage != samplv1_env::Release) {
+						m_dcf1.env.note_off_fast(&pv->dcf1_env);
+						m_lfo1.env.note_off_fast(&pv->lfo1_env);
+						m_dca1.env.note_off_fast(&pv->dca1_env);
+						m_notes[pv->note] = 0;
+						pv->note = -1;
+					}
+				}
+			}
+			pv = m_notes[key];
+			if (pv && pv->note >= 0/* && !m_ctl1.sustain*/) {
+				// retrigger fast release
+				m_dcf1.env.note_off_fast(&pv->dcf1_env);
+				m_lfo1.env.note_off_fast(&pv->lfo1_env);
+				m_dca1.env.note_off_fast(&pv->dca1_env);
+				m_notes[pv->note] = 0;
+				pv->note = -1;
+			}
+			// find free voice
+			pv = alloc_voice();
+			if (pv) {
+				// waveform
+				pv->note = key;
+				// velocity
+				const float vel = float(value) / 127.0f;
+				// quadratic velocity law
+				pv->vel = samplv1_velocity(vel * vel, *m_def.velocity);
+				// pressure/after-touch
+				pv->pre = 0.0f;
+				pv->dca1_pre.reset(m_def.pressure, &m_ctl1.pressure, &pv->pre);
+				// generate
+				pv->gen1.start();
+				// frequencies
+				const float freq1 = float(key)
+					+ *m_gen1.octave * OCTAVE_SCALE
+					+ *m_gen1.tuning * TUNING_SCALE;
+				pv->gen1_freq = samplv1_freq(freq1);
+				// filters
+				const int type1 = int(*m_dcf1.type);
+				pv->dcf11.reset(samplv1_filter1::Type(type1));
+				pv->dcf12.reset(samplv1_filter1::Type(type1));
+				pv->dcf13.reset(samplv1_filter2::Type(type1));
+				pv->dcf14.reset(samplv1_filter2::Type(type1));
+				// envelopes
+				m_dcf1.env.start(&pv->dcf1_env);
+				m_lfo1.env.start(&pv->lfo1_env);
+				m_dca1.env.start(&pv->dca1_env);
+				// lfos
+				pv->lfo1_sample = pv->lfo1.start();
+				// glides (portamentoa)
+				const float frames
+					= uint32_t(*m_gen1.glide * *m_gen1.glide * m_srate);
+				pv->gen1_glide.reset(frames, pv->gen1_freq);
+				// sustain
+				pv->sustain = false;
+				// allocated
+				m_notes[key] = pv;
+			}
+		}
+		// note off
+		else if (status == 0x80 || (status == 0x90 && value == 0)) {
+			samplv1_voice *pv = m_notes[key];
+			if (pv && pv->note >= 0) {
+				if (m_ctl1.sustain)
+					pv->sustain = true;
+				else
+				if (pv->dca1_env.stage != samplv1_env::Release) {
+					m_dca1.env.note_off(&pv->dca1_env);
+					m_dcf1.env.note_off(&pv->dcf1_env);
+					m_lfo1.env.note_off(&pv->lfo1_env);
+					pv->gen1.setLoop(false);
 				}
 			}
 		}
-		pv = m_notes[key];
-		if (pv && pv->note >= 0/* && !m_ctl.sustain*/) {
-			// retrigger fast release
-			m_dcf1.env.note_off_fast(&pv->dcf1_env);
-			m_lfo1.env.note_off_fast(&pv->lfo1_env);
-			m_dca1.env.note_off_fast(&pv->dca1_env);
-			m_notes[pv->note] = 0;
-			pv->note = -1;
+		// key pressure/poly.aftertouch
+		else if (status == 0xa0) {
+			samplv1_voice *pv = m_notes[key];
+			if (pv && pv->note >= 0)
+				pv->pre = *m_def.pressure * float(value) / 127.0f;
 		}
-		// find free voice
-		pv = alloc_voice();
-		if (pv) {
-			// waveform
-			pv->note = key;
-			// velocity
-			const float vel = float(value) / 127.0f;
-			// quadratic velocity law
-			pv->vel = samplv1_velocity(vel * vel, *m_def.velocity);
-			// pressure/after-touch
-			pv->pre = 0.0f;
-			pv->dca1_pre.reset(m_def.pressure, &m_ctl.pressure, &pv->pre);
-			// generate
-			pv->gen1.start();
-			// frequencies
-			const float freq1 = float(key)
-				+ *m_gen1.octave * OCTAVE_SCALE
-				+ *m_gen1.tuning * TUNING_SCALE;
-			pv->gen1_freq = samplv1_freq(freq1);
-			// filters
-			const int type1 = int(*m_dcf1.type);
-			pv->dcf11.reset(samplv1_filter1::Type(type1));
-			pv->dcf12.reset(samplv1_filter1::Type(type1));
-			pv->dcf13.reset(samplv1_filter2::Type(type1));
-			pv->dcf14.reset(samplv1_filter2::Type(type1));
-			// envelopes
-			m_dcf1.env.start(&pv->dcf1_env);
-			m_lfo1.env.start(&pv->lfo1_env);
-			m_dca1.env.start(&pv->dca1_env);
-			// lfos
-			pv->lfo1_sample = pv->lfo1.start();
-			// glides (portamentoa)
-			const float srate = float(m_iSampleRate);
-			float frames = uint32_t(*m_gen1.glide * *m_gen1.glide * srate);
-			pv->gen1_glide.reset(frames, pv->gen1_freq);
-			// sustain
-			pv->sustain = false;
-			// allocated
-			m_notes[key] = pv;
-		}
-	}
-	// note off
-	else if (status == 0x80 || (status == 0x90 && value == 0)) {
-		samplv1_voice *pv = m_notes[key];
-		if (pv && pv->note >= 0) {
-			if (m_ctl.sustain)
-				pv->sustain = true;
-			else
-			if (pv->dca1_env.stage != samplv1_env::Release) {
-				m_dca1.env.note_off(&pv->dca1_env);
-				m_dcf1.env.note_off(&pv->dcf1_env);
-				m_lfo1.env.note_off(&pv->lfo1_env);
-				pv->gen1.setLoop(false);
+		// control change
+		else if (status == 0xb0) {
+		switch (key) {
+			case 0x00:
+				// bank-select MSB (cc#0)
+				m_programs.bank_select_msb(value);
+				break;
+			case 0x01:
+				// modulation wheel (cc#1)
+				m_ctl1.modwheel = *m_def.modwheel * float(value) / 127.0f;
+				break;
+			case 0x07:
+				// channel volume (cc#7)
+				m_ctl1.volume = float(value) / 127.0f;
+				break;
+			case 0x0a:
+				// channel panning (cc#10)
+				m_ctl1.panning = float(value - 64) / 64.0f;
+				break;
+			case 0x20:
+				// bank-select LSB (cc#32)
+				m_programs.bank_select_lsb(value);
+				break;
+			case 0x40:
+				// sustain/damper pedal (cc#64)
+				if (m_ctl1.sustain && value <  64)
+					allSustainOff();
+				m_ctl1.sustain = bool(value >= 64);
+				break;
+			case 0x78:
+				// all sound off (cc#120)
+				allSoundOff();
+				break;
+			case 0x79:
+				// all controllers off (cc#121)
+				allControllersOff();
+				break;
+			case 0x7b:
+				// all notes off (cc#123)
+				allNotesOff();
+				break;
 			}
+			// process controllers...
+			m_controls.process_enqueue(channel, key, value);
 		}
-	}
-	// key pressure/poly.aftertouch
-	else if (status == 0xa0) {
-		samplv1_voice *pv = m_notes[key];
-		if (pv && pv->note >= 0)
-			pv->pre = *m_def.pressure * float(value) / 127.0f;
-	}
-	// control change
-	else if (status == 0xb0) {
-	switch (key) {
-		case 0x00:
-			// bank-select MSB (cc#0)
-			m_programs.bank_select_msb(value);
-			break;
-		case 0x01:
-			// modulation wheel (cc#1)
-			m_ctl.modwheel = *m_def.modwheel * float(value) / 127.0f;
-			break;
-		case 0x07:
-			// channel volume (cc#7)
-			m_ctl.volume = float(value) / 127.0f;
-			break;
-		case 0x0a:
-			// channel panning (cc#10)
-			m_ctl.panning = float(value - 64) / 64.0f;
-			break;
-		case 0x20:
-			// bank-select LSB (cc#32)
-			m_programs.bank_select_lsb(value);
-			break;
-		case 0x40:
-			// sustain/damper pedal (cc#64)
-			if (m_ctl.sustain && value <  64)
-				allSustainOff();
-			m_ctl.sustain = bool(value >= 64);
-			break;
-		case 0x78:
-			// all sound off (cc#120)
-			allSoundOff();
-			break;
-		case 0x79:
-			// all controllers off (cc#121)
-			allControllersOff();
-			break;
-		case 0x7b:
-			// all notes off (cc#123)
-			allNotesOff();
-			break;
+		// pitch bend
+		else if (status == 0xe0) {
+			const float pitchbend = float(key + (value << 7) - 0x2000) / 8192.0f;
+			m_ctl1.pitchbend = samplv1_pow2f(*m_def.pitchbend * pitchbend);
 		}
+
 	}
-	// pitch bend
-	else if (status == 0xe0) {
-		const float pitchbend = float(key + (value << 7) - 0x2000) / 8192.0f;
-		m_ctl.pitchbend = samplv1_pow2f(*m_def.pitchbend * pitchbend);
-	}
+
+	// process pending controllers...
+	m_controls.process_dequeue();
 }
 
 
@@ -1377,7 +1426,7 @@ void samplv1_impl::process_midi ( uint8_t *data, uint32_t size )
 
 void samplv1_impl::allControllersOff (void)
 {
-	m_ctl.reset();
+	m_ctl1.reset();
 }
 
 
@@ -1385,20 +1434,20 @@ void samplv1_impl::allControllersOff (void)
 
 void samplv1_impl::allSoundOff (void)
 {
-	m_chorus.setSampleRate(m_iSampleRate);
+	m_chorus.setSampleRate(m_srate);
 	m_chorus.reset();
 
-	for (uint16_t k = 0; k < m_iChannels; ++k) {
-		m_phaser[k].setSampleRate(m_iSampleRate);
-		m_delay[k].setSampleRate(m_iSampleRate);
-		m_comp[k].setSampleRate(m_iSampleRate);
+	for (uint16_t k = 0; k < m_nchannels; ++k) {
+		m_phaser[k].setSampleRate(m_srate);
+		m_delay[k].setSampleRate(m_srate);
+		m_comp[k].setSampleRate(m_srate);
 		m_flanger[k].reset();
 		m_phaser[k].reset();
 		m_delay[k].reset();
 		m_comp[k].reset();
 	}
 
-	m_reverb.setSampleRate(m_iSampleRate);
+	m_reverb.setSampleRate(m_srate);
 	m_reverb.reset();
 }
 
@@ -1445,7 +1494,7 @@ void samplv1_impl::allSustainOff (void)
 
 void samplv1_impl::reset (void)
 {
-#if 1//--legacy support < 0.3.0.4
+#if 0//--legacy support < 0.3.0.4
 	if (*m_del.bpm < 3.6f)
 		*m_del.bpm *= 100.0f;
 #endif
@@ -1454,35 +1503,46 @@ void samplv1_impl::reset (void)
 	m_del.bpmsync0 = 0.0f;
 	m_del.bpm0 = m_del.bpm;
 
-	m_vol1.reset(m_out1.volume, m_dca1.volume, &m_ctl.volume, &m_aux1.volume);
-	m_pan1.reset(m_out1.panning, &m_ctl.panning, &m_aux1.panning);
+	m_vol1.reset(m_out1.volume, m_dca1.volume, &m_ctl1.volume, &m_aux1.volume);
+	m_pan1.reset(m_out1.panning, &m_ctl1.panning, &m_aux1.panning);
 	m_wid1.reset(m_out1.width);
 
 	// flangers
 	if (m_flanger == 0)
-		m_flanger = new samplv1_fx_flanger [m_iChannels];
+		m_flanger = new samplv1_fx_flanger [m_nchannels];
 
 	// phasers
 	if (m_phaser == 0)
-		m_phaser = new samplv1_fx_phaser [m_iChannels];
+		m_phaser = new samplv1_fx_phaser [m_nchannels];
 
 	// delays
 	if (m_delay == 0)
-		m_delay = new samplv1_fx_delay [m_iChannels];
+		m_delay = new samplv1_fx_delay [m_nchannels];
 
 	// compressors
 	if (m_comp == 0)
-		m_comp = new samplv1_fx_comp [m_iChannels];
+		m_comp = new samplv1_fx_comp [m_nchannels];
 
 	// reverbs
 	m_reverb.reset();
 
+	// controllers reset.
+	m_controls.reset();
+
 	allSoundOff();
 //	allControllersOff();
 	allNotesOff();
 }
 
 
+// controllers accessor
+
+samplv1_controls *samplv1_impl::controls (void)
+{
+	return &m_controls;
+}
+
+
 // programs accessor
 
 samplv1_programs *samplv1_impl::programs (void)
@@ -1495,13 +1555,13 @@ samplv1_programs *samplv1_impl::programs (void)
 
 void samplv1_impl::process ( float **ins, float **outs, uint32_t nframes )
 {
-	float *v_outs[m_iChannels];
+	float *v_outs[m_nchannels];
 
 	// buffer i/o transfer
 
 	uint16_t k;
 
-	for (k = 0; k < m_iChannels; ++k)
+	for (k = 0; k < m_nchannels; ++k)
 		::memcpy(outs[k], ins[k], nframes * sizeof(float));
 
 	// channel indexes
@@ -1517,7 +1577,7 @@ void samplv1_impl::process ( float **ins, float **outs, uint32_t nframes )
 		= LFO_FREQ_MIN + lfo1_rate * (LFO_FREQ_MAX - LFO_FREQ_MIN);
 
 	const float modwheel1
-		= m_ctl.modwheel + PITCH_SCALE * *m_lfo1.pitch;
+		= m_ctl1.modwheel + PITCH_SCALE * *m_lfo1.pitch;
 
 	if (m_gen1.sample0 != *m_gen1.sample) {
 		m_gen1.sample0  = *m_gen1.sample;
@@ -1545,7 +1605,7 @@ void samplv1_impl::process ( float **ins, float **outs, uint32_t nframes )
 
 		// output buffers
 
-		for (k = 0; k < m_iChannels; ++k)
+		for (k = 0; k < m_nchannels; ++k)
 			v_outs[k] = outs[k];
 
 		uint32_t nblock = nframes;
@@ -1576,7 +1636,7 @@ void samplv1_impl::process ( float **ins, float **outs, uint32_t nframes )
 				const float lfo1 = pv->lfo1_sample * lfo1_env;
 
 				pv->gen1.next(pv->gen1_freq
-					* (m_ctl.pitchbend + modwheel1 * lfo1)
+					* (m_ctl1.pitchbend + modwheel1 * lfo1)
 					+ pv->gen1_glide.tick());
 
 				float gen1 = pv->gen1.value(k11);
@@ -1617,7 +1677,7 @@ void samplv1_impl::process ( float **ins, float **outs, uint32_t nframes )
 				const float out2
 					= vol1 * (mid1 - sid1 * wid1) * m_pan1.value(j, 1);
 
-				for (k = 0; k < m_iChannels; ++k)
+				for (k = 0; k < m_nchannels; ++k)
 					*v_outs[k]++ += (k & 1 ? out2 : out1);
 
 				if (j == 0) {
@@ -1663,13 +1723,13 @@ void samplv1_impl::process ( float **ins, float **outs, uint32_t nframes )
 
 
 	// chorus
-	if (m_iChannels > 1) {
+	if (m_nchannels > 1) {
 		m_chorus.process(outs[0], outs[1], nframes, *m_cho.wet,
 			*m_cho.delay, *m_cho.feedb, *m_cho.rate, *m_cho.mod);
 	}
 
 	// effects
-	for (k = 0; k < m_iChannels; ++k) {
+	for (k = 0; k < m_nchannels; ++k) {
 		float *in = outs[k];
 		// flanger
 		m_flanger[k].process(in, nframes, *m_fla.wet,
@@ -1683,13 +1743,13 @@ void samplv1_impl::process ( float **ins, float **outs, uint32_t nframes )
 	}
 
 	// reverb
-	if (m_iChannels > 1) {
+	if (m_nchannels > 1) {
 		m_reverb.process(outs[0], outs[1], nframes, *m_rev.wet,
 			*m_rev.feedb, *m_rev.room, *m_rev.damp, *m_rev.width);
 	}
 
 	// dynamics
-	for (k = 0; k < m_iChannels; ++k) {
+	for (k = 0; k < m_nchannels; ++k) {
 		float *in = outs[k];
 		// compressor
 		if (int(*m_dyn.compress) > 0)
@@ -1707,6 +1767,8 @@ void samplv1_impl::process ( float **ins, float **outs, uint32_t nframes )
 	m_wid1.process(nframes);
 	m_pan1.process(nframes);
 	m_vol1.process(nframes);
+
+	m_controls.process(nframes);
 }
 
 
@@ -1715,9 +1777,9 @@ void samplv1_impl::process ( float **ins, float **outs, uint32_t nframes )
 // samplv1 - decl.
 //
 
-samplv1::samplv1 ( uint16_t iChannels, uint32_t iSampleRate )
+samplv1::samplv1 ( uint16_t nchannels, float srate )
 {
-	m_pImpl = new samplv1_impl(this, iChannels, iSampleRate);
+	m_pImpl = new samplv1_impl(this, nchannels, srate);
 }
 
 
@@ -1727,9 +1789,9 @@ samplv1::~samplv1 (void)
 }
 
 
-void samplv1::setChannels ( uint16_t iChannels )
+void samplv1::setChannels ( uint16_t nchannels )
 {
-	m_pImpl->setChannels(iChannels);
+	m_pImpl->setChannels(nchannels);
 }
 
 
@@ -1739,13 +1801,13 @@ uint16_t samplv1::channels (void) const
 }
 
 
-void samplv1::setSampleRate ( uint32_t iSampleRate )
+void samplv1::setSampleRate ( float srate )
 {
-	m_pImpl->setSampleRate(iSampleRate);
+	m_pImpl->setSampleRate(srate);
 }
 
 
-uint32_t samplv1::sampleRate (void) const
+float samplv1::sampleRate (void) const
 {
 	return m_pImpl->sampleRate();
 }
@@ -1848,6 +1910,14 @@ void samplv1::process ( float **ins, float **outs, uint32_t nframes )
 }
 
 
+// controllers accessor
+
+samplv1_controls *samplv1::controls (void) const
+{
+	return m_pImpl->controls();
+}
+
+
 // programs accessor
 
 samplv1_programs *samplv1::programs (void) const
