@@ -29,6 +29,7 @@
 // forward declarations
 class samplv1_impl;
 class samplv1_sample;
+class samplv1_controls;
 class samplv1_programs;
 
 
@@ -40,15 +41,15 @@ class samplv1
 {
 public:
 
-	samplv1(uint16_t iChannels = 2, uint32_t iSampleRate = 44100);
+	samplv1(uint16_t nchannels = 2, float srate = 44100.0f);
 
 	~samplv1();
 
-	void setChannels(uint16_t iChannels);
+	void setChannels(uint16_t nchannels);
 	uint16_t channels() const;
 
-	void setSampleRate(uint32_t iSampleRate);
-	uint32_t sampleRate() const;
+	void setSampleRate(float srate);
+	float sampleRate() const;
 
 	void setSampleFile(const char *pszSampleFile);
 	const char *sampleFile() const;
@@ -151,6 +152,7 @@ public:
 
 	void reset();
 
+	samplv1_controls *controls() const;
 	samplv1_programs *programs() const;
 
 	void process_midi(uint8_t *data, uint32_t size);
