@@ -20,7 +20,9 @@
 *****************************************************************************/
 
 #include "samplv1_config.h"
+
 #include "samplv1_programs.h"
+#include "samplv1_controls.h"
 
 #include <QFileInfo>
 
@@ -148,11 +150,15 @@ void samplv1_config::loadPrograms ( samplv1_programs *pPrograms )
 	}
 
 	QSettings::endGroup();
+
+	pPrograms->enabled(bProgramsEnabled);
 }
 
 
 void samplv1_config::savePrograms ( samplv1_programs *pPrograms )
 {
+	bProgramsEnabled = pPrograms->enabled();
+
 	clearPrograms();
 
 	QSettings::beginGroup(programsGroup());
@@ -206,6 +212,97 @@ void samplv1_config::clearPrograms (void)
 }
 
 
+// Programs utility methods.
+QString samplv1_config::controlsGroup (void) const
+{
+	return "/Controllers";
+}
+
+QString samplv1_config::controlPrefix (void) const
+{
+	return "/Control";
+}
+
+
+void samplv1_config::loadControls ( samplv1_controls *pControls )
+{
+	pControls->clear();
+
+	QSettings::beginGroup(controlsGroup());
+
+	const QStringList& keys = QSettings::childKeys();
+	QStringListIterator iter(keys);
+	while (iter.hasNext()) {
+		const QString& sKey = '/' + iter.next();
+		const QStringList& clist = sKey.split('_');
+		if (clist.at(0) == controlPrefix()) {
+			const unsigned short channel
+				= clist.at(1).toInt();
+			const samplv1_controls::Type ctype
+				= samplv1_controls::typeFromText(clist.at(2));
+			samplv1_controls::Key key;
+			key.status = ctype | (channel & 0x1f);
+			key.param = clist.at(3).toInt();
+			const QStringList& vlist
+				= QSettings::value(sKey).toStringList();
+			samplv1_controls::Data data;
+			data.index = vlist.at(0).toInt();
+			if (vlist.count() > 1)
+				data.flags = vlist.at(1).toInt();
+			pControls->add_control(key, data);
+		}
+	}
+
+	QSettings::endGroup();
+
+	pControls->enabled(bControlsEnabled);
+}
+
+
+void samplv1_config::saveControls ( samplv1_controls *pControls )
+{
+	bControlsEnabled = pControls->enabled();
+
+	clearControls();
+
+	QSettings::beginGroup(controlsGroup());
+
+	const samplv1_controls::Map& map = pControls->map();
+	samplv1_controls::Map::ConstIterator iter = map.constBegin();
+	const samplv1_controls::Map::ConstIterator& iter_end = map.constEnd();
+	for ( ; iter != iter_end; ++iter) {
+		const samplv1_controls::Key& key = iter.key();
+		QString sKey = controlPrefix();
+		sKey += '_' + QString::number(key.channel());
+		sKey += '_' + samplv1_controls::textFromType(key.type());
+		sKey += '_' + QString::number(key.param);
+		const samplv1_controls::Data& data = iter.value();
+		QStringList vlist;
+		vlist.append(QString::number(data.index));
+		vlist.append(QString::number(data.flags));
+		QSettings::setValue(sKey, vlist);
+	}
+
+	QSettings::endGroup();
+	QSettings::sync();
+}
+
+
+void samplv1_config::clearControls (void)
+{
+	QSettings::beginGroup(controlsGroup());
+
+	const QStringList& keys = QSettings::childKeys();
+	QStringListIterator iter(keys);
+	while (iter.hasNext()) {
+		const QString& key = iter.next();
+		QSettings::remove(key);
+	}
+
+	QSettings::endGroup();
+}
+
+
 // Explicit I/O methods.
 void samplv1_config::load (void)
 {
@@ -214,6 +311,8 @@ void samplv1_config::load (void)
 	sPresetDir = QSettings::value("/PresetDir").toString();
 	sSampleDir = QSettings::value("/SampleDir").toString();
 	iKnobDialMode = QSettings::value("/KnobDialMode", 0).toInt();
+	bControlsEnabled = QSettings::value("/ControlsEnabled", false).toBool();
+	bProgramsEnabled = QSettings::value("/ProgramsEnabled", false).toBool();
 	QSettings::endGroup();
 
 	QSettings::beginGroup("/Dialogs");
@@ -240,6 +339,8 @@ void samplv1_config::save (void)
 	QSettings::setValue("/PresetDir", sPresetDir);
 	QSettings::setValue("/SampleDir", sSampleDir);
 	QSettings::setValue("/KnobDialMode", iKnobDialMode);
+	QSettings::setValue("/ControlsEnabled", bControlsEnabled);
+	QSettings::setValue("/ProgramsEnabled", bProgramsEnabled);
 	QSettings::endGroup();
 
 	QSettings::beginGroup("/Dialogs");
