@@ -43,6 +43,7 @@
 
 // forward decls.
 class samplv1_programs;
+class samplv1_controls;
 
 
 class samplv1_config : public QSettings
@@ -64,6 +65,8 @@ public:
 	int iKnobDialMode;
 
 	// Special persistent options.
+	bool bControlsEnabled;
+	bool bProgramsEnabled;
 	bool bProgramsPreview;
 	bool bUseNativeDialogs;
 	// Run-time special non-persistent options.
@@ -85,6 +88,10 @@ public:
 	void loadPrograms(samplv1_programs *pPrograms);
 	void savePrograms(samplv1_programs *pPrograms);
 
+	// Controllers utility methods.
+	void loadControls(samplv1_controls *pControls);
+	void saveControls(samplv1_controls *pControls);
+
 protected:
 
 	// Preset group path.
@@ -96,6 +103,12 @@ protected:
 
 	void clearPrograms();
 
+	// Controllers group path.
+	QString controlsGroup() const;
+	QString controlPrefix() const;
+
+	void clearControls();
+
 	// Explicit I/O methods.
 	void load();
 	void save();
