@@ -23,6 +23,9 @@
 #include "samplv1_config.h"
 #include "samplv1_param.h"
 
+#include "samplv1_programs.h"
+#include "samplv1_controls.h"
+
 #include <jack/midiport.h>
 
 #include <stdio.h>
@@ -163,6 +166,9 @@ samplv1_jack::samplv1_jack (void) : samplv1(2)
 
 	m_bpm = 0.0f;
 
+	samplv1::programs()->enabled(true);
+	samplv1::controls()->enabled(true);
+
 	open(SAMPLV1_TITLE);
 	activate();
 }
@@ -272,7 +278,7 @@ void samplv1_jack::open ( const char *client_id )
 {
 	// init param ports
 	for (uint32_t i = 0; i < samplv1::NUM_PARAMS; ++i) {
-		samplv1::ParamIndex index = samplv1::ParamIndex(i);
+		const samplv1::ParamIndex index = samplv1::ParamIndex(i);
 		m_params[i] = samplv1_param::paramDefaultValue(index);
 		samplv1::setParamPort(index, &m_params[i]);
 	}
@@ -283,7 +289,7 @@ void samplv1_jack::open ( const char *client_id )
 		return;
 
 	// set sample rate
-	samplv1::setSampleRate(jack_get_sample_rate(m_client));
+	samplv1::setSampleRate(float(jack_get_sample_rate(m_client)));
 //	samplv1::reset();
 
 	// register audio ports & buffers
