@@ -23,6 +23,9 @@
 
 #include "samplv1_sched.h"
 
+#include "samplv1_programs.h"
+#include "samplv1_controls.h"
+
 #include "lv2/lv2plug.in/ns/ext/midi/midi.h"
 #include "lv2/lv2plug.in/ns/ext/time/time.h"
 #include "lv2/lv2plug.in/ns/ext/atom/util.h"
@@ -39,7 +42,7 @@
 
 samplv1_lv2::samplv1_lv2 (
 	double sample_rate, const LV2_Feature *const *host_features )
-	: samplv1(2, uint32_t(sample_rate))
+	: samplv1(2, float(sample_rate))
 {
 	m_urid_map = NULL;
 	m_atom_sequence = NULL;
@@ -70,6 +73,9 @@ samplv1_lv2::samplv1_lv2 (
 	m_outs = new float * [nchannels];
 	for (uint16_t k = 0; k < nchannels; ++k)
 		m_ins[k] = m_outs[k] = NULL;
+
+	samplv1::programs()->optional(true);
+	samplv1::controls()->optional(true);
 }
 
 
@@ -336,7 +342,7 @@ static LV2_State_Status samplv1_lv2_state_restore ( LV2_Handle instance,
 	if (loop_start < loop_end)
 		pPlugin->setLoopRange(loop_start, loop_end);
 
-	samplv1_sched::sync_notify(samplv1_sched::Sample);
+	samplv1_sched::sync_notify(pPlugin, samplv1_sched::Sample, 1);
 
 	return LV2_STATE_SUCCESS;
 }
