@@ -1,7 +1,7 @@
 // samplv1_lv2.h
 //
 /****************************************************************************
-   Copyright (C) 2012-2014, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2015, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -32,6 +32,12 @@
 #define SAMPLV1_LV2_PREFIX SAMPLV1_LV2_URI "#"
 
 
+#ifdef CONFIG_LV2_PROGRAMS
+#include "lv2_programs.h"
+#include <QByteArray>
+#endif
+
+
 //-------------------------------------------------------------------------
 // samplv1_lv2 - decl.
 //
@@ -63,6 +69,11 @@ public:
 
 	uint32_t urid_map(const char *uri) const;
 
+#ifdef CONFIG_LV2_PROGRAMS
+	const LV2_Program_Descriptor *get_program(uint32_t index);
+	void select_program(uint32_t bank, uint32_t program);
+#endif
+
 private:
 
 	LV2_URID_Map *m_urid_map;
@@ -82,6 +93,11 @@ private:
 
 	float **m_ins;
 	float **m_outs;
+
+#ifdef CONFIG_LV2_PROGRAMS
+	LV2_Program_Descriptor m_program;
+	QByteArray m_aProgramName;
+#endif
 };
 
 
