@@ -26,6 +26,7 @@
 #include "samplv1_sched.h"
 
 #include "samplv1widget_config.h"
+#include "samplv1widget_control.h"
 
 #include <QMessageBox>
 #include <QDir>
@@ -63,7 +64,7 @@ samplv1widget::samplv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 	m_ui.setupUi(this);
 
 	// Init sched notifier.
-	m_sched_notifier = new samplv1_sched_notifier(this);
+	m_sched_notifier = NULL;
 
 	// Init swapable params A/B to default.
 	for (uint32_t i = 0; i < samplv1::NUM_PARAMS; ++i)
@@ -489,11 +490,6 @@ samplv1widget::samplv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 		SIGNAL(triggered(bool)),
 		SLOT(helpAboutQt()));
 
-	// Special sample update notifications (eg. reverse)
-	QObject::connect(m_sched_notifier,
-		SIGNAL(notify(int)),
-		SLOT(updateSchedNotify(int)));
-
 	// General knob/dial  behavior init...
 	samplv1_config *pConfig = samplv1_config::getInstance();
 	if (pConfig) {
@@ -513,7 +509,28 @@ samplv1widget::samplv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 // Destructor.
 samplv1widget::~samplv1widget (void)
 {
-	delete m_sched_notifier;
+	if (m_sched_notifier)
+		delete m_sched_notifier;
+}
+
+
+// Create/initialize the scheduler/work notifier.
+void samplv1widget::initSchedNotifier (void)
+{
+	if (m_sched_notifier) {
+		delete m_sched_notifier;
+		m_sched_notifier = NULL;
+	}
+
+	samplv1_ui *pSamplUi = ui_instance();
+	if (pSamplUi == NULL)
+		return;
+
+	m_sched_notifier = new samplv1widget_sched(pSamplUi->instance(), this);
+
+	QObject::connect(m_sched_notifier,
+		SIGNAL(notify(int, int)),
+		SLOT(updateSchedNotify(int, int)));
 }
 
 
@@ -528,6 +545,12 @@ void samplv1widget::setParamKnob ( samplv1::ParamIndex index, samplv1widget_knob
 	QObject::connect(pKnob,
 		SIGNAL(valueChanged(float)),
 		SLOT(paramChanged(float)));
+
+	pKnob->setContextMenuPolicy(Qt::CustomContextMenu);
+
+	QObject::connect(pKnob,
+		SIGNAL(customContextMenuRequested(const QPoint&)),
+		SLOT(paramContextMenu(const QPoint&)));
 }
 
 samplv1widget_knob *samplv1widget::paramKnob ( samplv1::ParamIndex index ) const
@@ -576,11 +599,10 @@ void samplv1widget::paramChanged ( float fValue )
 
 	samplv1widget_knob *pKnob = qobject_cast<samplv1widget_knob *> (sender());
 	if (pKnob) {
-		samplv1::ParamIndex index = m_knobParams.value(pKnob);
+		const samplv1::ParamIndex index = m_knobParams.value(pKnob);
 		updateParam(index, fValue);
 		updateParamEx(index, fValue);
-		m_ui.StatusBar->showMessage(QString("%1 / %2: %3")
-			.arg(m_ui.StackedWidget->currentWidget()->windowTitle())
+		m_ui.StatusBar->showMessage(QString("%1: %2")
 			.arg(pKnob->toolTip())
 			.arg(pKnob->valueText()), 5000);
 		updateDirtyPreset(true);
@@ -598,7 +620,7 @@ void samplv1widget::updateParamEx ( samplv1::ParamIndex index, float fValue )
 	++m_iUpdate;
 
 	switch (index) {
-#if 1//--updateSchedNotify(samplv1_sched::Sample);
+#if 1//--updateSchedNotify(samplv1_sched::Sample, 0);
 	case samplv1::GEN1_REVERSE: {
 		const bool bReverse = bool(fValue > 0.0f);
 		pSamplUi->setReverse(bReverse);
@@ -628,6 +650,26 @@ void samplv1widget::updateParamEx ( samplv1::ParamIndex index, float fValue )
 }
 
 
+// Update scheduled controllers param/knob widgets.
+void samplv1widget::updateSchedParam ( samplv1::ParamIndex index, float fValue )
+{
+	++m_iUpdate;
+
+	samplv1widget_knob *pKnob = paramKnob(index);
+	if (pKnob) {
+		pKnob->setValue(fValue, false);
+		updateParam(index, fValue);
+		updateParamEx(index, fValue);
+		m_ui.StatusBar->showMessage(QString("%1: %2")
+			.arg(pKnob->toolTip())
+			.arg(pKnob->valueText()), 5000);
+		updateDirtyPreset(true);
+	}
+
+	--m_iUpdate;
+}
+
+
 // Reset all param knobs to default values.
 void samplv1widget::resetParams (void)
 {
@@ -640,7 +682,7 @@ void samplv1widget::resetParams (void)
 	resetSwapParams();
 
 	for (uint32_t i = 0; i < samplv1::NUM_PARAMS; ++i) {
-		samplv1::ParamIndex index = samplv1::ParamIndex(i);
+		const samplv1::ParamIndex index = samplv1::ParamIndex(i);
 		float fValue = samplv1_param::paramDefaultValue(index);
 		samplv1widget_knob *pKnob = paramKnob(index);
 		if (pKnob)
@@ -667,7 +709,7 @@ void samplv1widget::swapParams ( bool bOn )
 //	resetParamKnobs();
 
 	for (uint32_t i = 0; i < samplv1::NUM_PARAMS; ++i) {
-		samplv1::ParamIndex index = samplv1::ParamIndex(i);
+		const samplv1::ParamIndex index = samplv1::ParamIndex(i);
 		samplv1widget_knob *pKnob = paramKnob(index);
 		if (pKnob) {
 			const float fOldValue = pKnob->value();
@@ -701,7 +743,7 @@ void samplv1widget::updateParamValues (void)
 	samplv1_ui *pSamplUi = ui_instance();
 
 	for (uint32_t i = 0; i < samplv1::NUM_PARAMS; ++i) {
-		samplv1::ParamIndex index = samplv1::ParamIndex(i);
+		const samplv1::ParamIndex index = samplv1::ParamIndex(i);
 		const float fValue = (pSamplUi
 			? pSamplUi->paramValue(index)
 			: samplv1_param::paramDefaultValue(index));
@@ -719,7 +761,7 @@ void samplv1widget::resetParamValues (void)
 	resetSwapParams();
 
 	for (uint32_t i = 0; i < samplv1::NUM_PARAMS; ++i) {
-		samplv1::ParamIndex index = samplv1::ParamIndex(i);
+		const samplv1::ParamIndex index = samplv1::ParamIndex(i);
 		const float fValue = samplv1_param::paramDefaultValue(index);
 		setParamValue(index, fValue, true);
 		updateParam(index, fValue);
@@ -880,11 +922,13 @@ void samplv1widget::updateSample ( samplv1_sample *pSample, bool bDirty )
 		const uint32_t iLoopEnd = pSample->loopEnd();
 		m_ui.Gen1Sample->setLoopStart(iLoopStart);
 		m_ui.Gen1Sample->setLoopEnd(iLoopEnd);
+		activateParamKnobs(pSample->filename() != NULL);
 		updateSampleLoop(pSample);
 	} else {
 		m_ui.Gen1Sample->setLoop(false);
 		m_ui.Gen1Sample->setLoopStart(0);
 		m_ui.Gen1Sample->setLoopEnd(0);
+		activateParamKnobs(false);
 		updateSampleLoop(NULL);
 	}
 	--m_iUpdate;
@@ -996,6 +1040,30 @@ void samplv1widget::updateSampleLoop ( samplv1_sample *pSample, bool bDirty )
 }
 
 
+// (En|Dis)able all param/knobs.
+void samplv1widget::activateParamKnobs ( bool bEnabled )
+{
+	activateParamKnobsGroupBox(m_ui.Gen1GroupBox, bEnabled);
+	activateParamKnobsGroupBox(m_ui.Dcf1GroupBox, bEnabled);
+	activateParamKnobsGroupBox(m_ui.Lfo1GroupBox, bEnabled);
+	activateParamKnobsGroupBox(m_ui.Dca1GroupBox, bEnabled);
+	activateParamKnobsGroupBox(m_ui.Out1GroupBox, bEnabled);
+
+	m_ui.Gen1Sample->setEnabled(true);
+}
+
+
+void samplv1widget::activateParamKnobsGroupBox (
+	QGroupBox *pGroupBox, bool bEnabled )
+{
+	const QList<QWidget *>& children
+		= pGroupBox->findChildren<QWidget *> ();
+	QListIterator<QWidget *> iter(children);
+	while (iter.hasNext())
+		iter.next()->setEnabled(bEnabled);
+}
+
+
 // Delay BPM change.
 void samplv1widget::bpmSyncChanged (void)
 {
@@ -1054,17 +1122,31 @@ void samplv1widget::updateLoadPreset ( const QString& sPreset )
 
 
 // Notification updater.
-void samplv1widget::updateSchedNotify ( int stype )
+void samplv1widget::updateSchedNotify ( int stype, int sid )
 {
 	samplv1_ui *pSamplUi = ui_instance();
 	if (pSamplUi == NULL)
 		return;
 
 #ifdef CONFIG_DEBUG
-	qDebug("samplv1widget::updateSchedNotify(%d)", stype);
+	qDebug("samplv1widget::updateSchedNotify(%d, %d)", stype, sid);
 #endif
 
 	switch (samplv1_sched::Type(stype)) {
+	case samplv1_sched::Controller: {
+		samplv1widget_control *pInstance
+			= samplv1widget_control::getInstance();
+		if (pInstance) {
+			samplv1_controls *pControls = pSamplUi->controls();
+			pInstance->setControlKey(pControls->current_key());
+		}
+		break;
+	}
+	case samplv1_sched::Controls: {
+		const samplv1::ParamIndex index = samplv1::ParamIndex(sid);
+		updateSchedParam(index, pSamplUi->paramValue(index));
+		break;
+	}
 	case samplv1_sched::Programs: {
 		samplv1_programs *pPrograms = pSamplUi->programs();
 		samplv1_programs::Prog *pProg = pPrograms->current_prog();
@@ -1073,6 +1155,10 @@ void samplv1widget::updateSchedNotify ( int stype )
 	}
 	case samplv1_sched::Sample:
 		updateSample(pSamplUi->sample());
+		if (sid > 0) {
+			updateParamValues();
+			updateDirtyPreset(false);
+		}
 		// Fall thru...
 	default:
 		break;
@@ -1089,8 +1175,10 @@ void samplv1widget::helpConfigure (void)
 
 	samplv1widget_config form(this);
 
-	// Set programs database...
+	// Set controllers&&programs database...
+	form.setControls(pSamplUi->controls());
 	form.setPrograms(pSamplUi->programs());
+
 	form.exec();
 }
 
@@ -1159,4 +1247,37 @@ void samplv1widget::updateDirtyPreset ( bool bDirtyPreset )
 }
 
 
+// Param knob context menu.
+void samplv1widget::paramContextMenu ( const QPoint& pos )
+{
+	samplv1widget_knob *pKnob
+		= qobject_cast<samplv1widget_knob *> (sender());
+	if (pKnob == NULL)
+		return;
+
+	samplv1_ui *pSamplUi = ui_instance();
+	if (pSamplUi == NULL)
+		return;
+
+	samplv1_controls *pControls = pSamplUi->controls();
+	if (pControls == NULL)
+		return;
+
+	if (!pControls->enabled())
+		return;
+
+	QMenu menu(this);
+
+	QAction *pAction = menu.addAction(
+		QIcon(":/images/samplv1_control.png"),
+		tr("MIDI &Controller..."));
+
+	if (menu.exec(pKnob->mapToGlobal(pos)) == pAction) {
+		const samplv1::ParamIndex index = m_knobParams.value(pKnob);
+		const QString& sTitle = pKnob->toolTip();
+		samplv1widget_control::showInstance(pControls, index, sTitle, this);
+	}
+}
+
+
 // end of samplv1widget.cpp
