@@ -25,12 +25,13 @@
 #include "ui_samplv1widget.h"
 
 #include "samplv1_config.h"
+#include "samplv1_sched.h"
 
 #include "samplv1_ui.h"
 
 
 // forward decls.
-class samplv1_sched_notifier;
+class samplv1widget_sched;
 
 
 //-------------------------------------------------------------------------
@@ -47,7 +48,10 @@ public:
 	samplv1widget(QWidget *pParent = 0, Qt::WindowFlags wflags = 0);
 
 	// Destructor.
-	~samplv1widget();
+	virtual ~samplv1widget();
+
+	// Create/initialize the scheduler/work notifier.
+	void initSchedNotifier();
 
 	// Param port accessors.
 	void setParamValue(
@@ -110,7 +114,10 @@ protected slots:
 	void bpmSyncChanged();
 
 	// Notification updater.
-	void updateSchedNotify(int stype);
+	void updateSchedNotify(int stype, int sid);
+
+	// Param knob context menu.
+	void paramContextMenu(const QPoint& pos);
 
 	// Menu actions.
 	void helpConfigure();
@@ -133,11 +140,9 @@ protected:
 	void resetParamValues();
 	void resetParamKnobs();
 
-	// Param port methods.
-	virtual void updateParam(samplv1::ParamIndex index, float fValue) const = 0;
-
-	// Update local tied widgets.
-	void updateParamEx(samplv1::ParamIndex index, float fValue);
+	// (En|Dis)able all param/knobs.
+	void activateParamKnobs(bool bEnabled);
+	void activateParamKnobsGroupBox(QGroupBox *pGroupBox, bool bEnable);
 
 	// Sample file clearance.
 	void clearSampleFile();
@@ -151,6 +156,15 @@ protected:
 	// Update loop range change status.
 	void updateSampleLoop(samplv1_sample *pSample, bool bDirty = false);
 
+	// Param port methods.
+	virtual void updateParam(samplv1::ParamIndex index, float fValue) const = 0;
+
+	// Update local tied widgets.
+	void updateParamEx(samplv1::ParamIndex index, float fValue);
+
+	// Update scheduled controllers param/knob widgets.
+	void updateSchedParam(samplv1::ParamIndex index, float fValue);
+
 	// Preset status updater.
 	void updateLoadPreset(const QString& sPreset);
 
@@ -162,7 +176,7 @@ private:
 	// Instance variables.
 	Ui::samplv1widget m_ui;
 
-	samplv1_sched_notifier *m_sched_notifier;
+	samplv1widget_sched *m_sched_notifier;
 
 	QHash<samplv1::ParamIndex, samplv1widget_knob *> m_paramKnobs;
 	QHash<samplv1widget_knob *, samplv1::ParamIndex> m_knobParams;
@@ -173,6 +187,54 @@ private:
 };
 
 
+//-------------------------------------------------------------------------
+// samplv1widget_sched - worker/schedule proxy decl.
+//
+
+class samplv1widget_sched : public QObject
+{
+	Q_OBJECT
+
+public:
+
+	// ctor.
+	samplv1widget_sched(samplv1 *pSampl, QObject *pParent = NULL)
+		: QObject(pParent), m_notifier(pSampl, this) {}
+
+signals:
+
+	// Notification signal.
+	void notify(int stype, int sid);
+
+protected:
+
+	// Notififier visitor.
+	class Notifier : public samplv1_sched_notifier
+	{
+	public:
+
+		Notifier(samplv1 *pSampl, samplv1widget_sched *pSched)
+			: samplv1_sched_notifier(pSampl), m_pSched(pSched) {}
+
+		void notify(samplv1_sched::Type stype, int sid) const
+			{ m_pSched->emit_notify(stype, sid); }
+
+	private:
+
+		samplv1widget_sched *m_pSched;
+	};
+
+	// Notification method.
+	void emit_notify(samplv1_sched::Type stype, int sid)
+		{ emit notify(int(stype), sid); }
+
+private:
+
+	// Instance variables.
+	Notifier m_notifier;
+};
+
+
 #endif	// __samplv1widget_h
 
 // end of samplv1widget.h
