@@ -1,7 +1,7 @@
 // samplv1widget_env.cpp
 //
 /****************************************************************************
-   Copyright (C) 2012-2013, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2015, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -163,6 +163,8 @@ void samplv1widget_env::paintEvent ( QPaintEvent *pPaintEvent )
 	grad.setColorAt(0.0f, rgbLite);
 	grad.setColorAt(1.0f, Qt::black);
 
+	painter.setRenderHint(QPainter::Antialiasing, true);
+
 	painter.setBrush(grad);
 	painter.drawPath(path);
 
@@ -184,6 +186,8 @@ void samplv1widget_env::paintEvent ( QPaintEvent *pPaintEvent )
 		.arg(int(100.0f * release())));
 #endif
 
+	painter.setRenderHint(QPainter::Antialiasing, false);
+
 	painter.end();
 
 	QFrame::paintEvent(pPaintEvent);
@@ -218,15 +222,16 @@ int samplv1widget_env::nodeIndex ( const QPoint& pos ) const
 
 void samplv1widget_env::dragNode ( const QPoint& pos )
 {
-	int h  = height();
-	int w  = width();
+	const int h  = height();
+	const int w  = width();
 
-	int w4 = (w - 12) >> 2;
+	const int w4 = (w - 12) >> 2;
 
-	int x, dx = (pos.x() - m_posDrag.x());
-	int y, dy = (pos.y() - m_posDrag.y());
+	const int dx = (pos.x() - m_posDrag.x());
+	const int dy = (pos.y() - m_posDrag.y());
 
 	if (dx || dy) {
+		int x, y;
 		switch (m_iDragNode) {
 		case 2: // Attack
 			x = int(attack() * float(w4));
@@ -244,10 +249,9 @@ void samplv1widget_env::dragNode ( const QPoint& pos )
 			x = int(release() * float(w4));
 			setRelease(float(x + dx) / float(w4));
 			break;
-		default:
-			break;
 		}
-		m_posDrag = pos;
+		m_posDrag = m_poly.at(m_iDragNode);
+	//	m_posDrag = pos;
 	}
 }
 
@@ -257,7 +261,7 @@ void samplv1widget_env::mousePressEvent ( QMouseEvent *pMouseEvent )
 {
 	if (pMouseEvent->button() == Qt::LeftButton) {
 		const QPoint& pos = pMouseEvent->pos();
-		int iDragNode = nodeIndex(pos);
+		const int iDragNode = nodeIndex(pos);
 		if (iDragNode >= 0) {
 			switch (iDragNode) {
 			case 2: // Attack
@@ -298,9 +302,8 @@ void samplv1widget_env::mouseReleaseEvent ( QMouseEvent *pMouseEvent )
 {
 	QFrame::mouseReleaseEvent(pMouseEvent);
 
-	dragNode(pMouseEvent->pos());
-
 	if (m_iDragNode >= 0) {
+		dragNode(pMouseEvent->pos());
 		m_iDragNode = -1;
 		unsetCursor();
 	}
