@@ -1,7 +1,7 @@
 // samplv1widget_filt.cpp
 //
 /****************************************************************************
-   Copyright (C) 2012-2013, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2015, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -140,7 +140,7 @@ void samplv1widget_filt::paintEvent ( QPaintEvent *pPaintEvent )
 	QPolygon poly(6);
 	QPainterPath path;
 
-	int iType = int(m_fType);
+	const int iType = int(m_fType);
 	// Low, Notch
 	if (iType == 0 || iType == 3) {
 		if (iType == 3) x -= w8;
@@ -203,6 +203,8 @@ void samplv1widget_filt::paintEvent ( QPaintEvent *pPaintEvent )
 	grad.setColorAt(0.0f, rgbLite);
 	grad.setColorAt(1.0f, Qt::black);
 
+	painter.setRenderHint(QPainter::Antialiasing, true);
+
 	painter.setBrush(grad);
 	painter.drawPath(path);
 
@@ -214,6 +216,8 @@ void samplv1widget_filt::paintEvent ( QPaintEvent *pPaintEvent )
 		.arg(int(100.0f * reso())));
 #endif
 
+	painter.setRenderHint(QPainter::Antialiasing, false);
+
 	painter.end();
 
 	QFrame::paintEvent(pPaintEvent);
@@ -223,16 +227,16 @@ void samplv1widget_filt::paintEvent ( QPaintEvent *pPaintEvent )
 // Drag/move curve.
 void samplv1widget_filt::dragCurve ( const QPoint& pos )
 {
-	int h  = height();
-	int w  = width();
+	const int h  = height();
+	const int w  = width();
 
-	int dx = (pos.x() - m_posDrag.x());
-	int dy = (pos.y() - m_posDrag.y());
+	const int dx = (pos.x() - m_posDrag.x());
+	const int dy = (pos.y() - m_posDrag.y());
 
 	if (dx || dy) {
-		int h2 = (h >> 1);
-		int x = int(cutoff() * float(w));
-		int y = int(reso() * float(h2));
+		const int h2 = (h >> 1);
+		const int x = int(cutoff() * float(w));
+		const int y = int(reso() * float(h2));
 		setCutoff(float(x + dx) / float(w));
 		setReso(float(y - dy) / float(h2));
 		m_posDrag = pos;
@@ -276,15 +280,15 @@ void samplv1widget_filt::mouseReleaseEvent ( QMouseEvent *pMouseEvent )
 
 void samplv1widget_filt::wheelEvent ( QWheelEvent *pWheelEvent )
 {
-	int delta = (pWheelEvent->delta() / 60);
+	const int delta = (pWheelEvent->delta() / 60);
 
 	if (pWheelEvent->modifiers() & (Qt::ShiftModifier | Qt::ControlModifier)) {
-		int h2 = (height() >> 1);
-		int y = int(reso() * float(h2));
+		const int h2 = (height() >> 1);
+		const int y = int(reso() * float(h2));
 		setReso(float(y + delta) / float(h2));
 	} else {
-		int w2 = (width() >> 1);
-		int x = int(cutoff() * float(w2));
+		const int w2 = (width() >> 1);
+		const int x = int(cutoff() * float(w2));
 		setCutoff(float(x + delta) / float(w2));
 	}
 }
