@@ -1,7 +1,7 @@
 // samplv1widget_jack.h
 //
 /****************************************************************************
-   Copyright (C) 2012-2014, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2015, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -44,6 +44,9 @@ public:
 	// Constructor.
 	samplv1widget_jack(samplv1_jack *pSampl);
 
+	// Destructor.
+	~samplv1widget_jack();
+
 #ifdef CONFIG_NSM
 	// NSM client accessors.
 	void setNsmClient(samplv1_nsm *pNsmClient);
@@ -53,7 +56,7 @@ public:
 protected:
 
 	// Synth engine accessor.
-	samplv1 *instance() const;
+	samplv1_ui *ui_instance() const;
 
 	// Param port method.
 	void updateParam(samplv1::ParamIndex index, float fValue) const;
@@ -73,7 +76,8 @@ protected:
 private:
 
 	// Instance variables.
-	samplv1_jack *m_pSampl;
+	samplv1     *m_pSampl;
+	samplv1_ui  *m_pSamplUi;
 
 #ifdef CONFIG_NSM
 	samplv1_nsm *m_pNsmClient;
