@@ -48,6 +48,9 @@ samplv1widget_lv2::samplv1widget_lv2 ( samplv1_lv2 *pSampl,
 	for (uint32_t i = 0; i < samplv1::NUM_PARAMS; ++i)
 		m_params_def[i] = true;
 
+	// May initialize the scheduler/work notifier.
+	initSchedNotifier();
+
 	// Initial update, always...
 	if (m_pSamplUi->sampleFile())
 		updateSample(m_pSamplUi->sample());
@@ -121,13 +124,13 @@ void samplv1widget_lv2::port_event ( uint32_t port_index,
 	uint32_t buffer_size, uint32_t format, const void *buffer )
 {
 	if (format == 0 && buffer_size == sizeof(float)) {
-		samplv1::ParamIndex index
+		const samplv1::ParamIndex index
 			= samplv1::ParamIndex(port_index - samplv1_lv2::ParamBase);
-		float fValue = *(float *) buffer;
-	//--legacy support < 0.3.0.4 -- begin
+		const float fValue = *(float *) buffer;
+	#if 0//--legacy support < 0.3.0.4 -- begin
 		if (index == samplv1::DEL1_BPM && fValue < 3.6f)
 			fValue *= 100.0f;
-	//--legacy support < 0.3.0.4 -- end.
+	#endif
 		setParamValue(index, fValue, m_params_def[index]);
 		m_params_def[index] = false;
 	}
